/***************************************************************************************
|---------------------------------------------------------------------------------------|
| Programmbeschreibung: | Mit diesem Programm kann das DCF-Signal mit einer             |
|                       | DCF-Antenne eingelesen werden.                                |
|---------------------------------------------------------------------------------------|
| Autor:                | Marcel Kaptein.                                               |
|                       | Wobei die Grundstruktur dieses Programmes auf einem Progamm   |
|                       | der Internetseite >> www.digital-hobby.de << basiert.         |
|                       | Das Programm auf der Internetseite funktionierte aber leider  |
|                       | nicht.                                                        |
|-----------------------|---------------------------------------------------------------|
| Besonderheiten:       | In diesem Programm gibt es, im vergleich zu dem im Internet,  |
|                       | noch einige ander, zustzliche Funktionen.                    |
|                       | Es wird z.B. auch der Tag, Monat und das Jahr eingelesen.     |
|                       | Ebenfalls gibt es eine Korrektur fr die Uhr, falls sie ohne  |
|                       | dem DCF-Signal luft. ( kein Empfang! )                       |
|                       | Fr den Uhr-Status werden auf dem Display drei verschiedene   |
|                       | Symbole angezeigt. Fr: Uhr luft mit oder ohne               |
|                       | Synkronisation, oder das einlesen des Signales ist gerade     |
|                       | in arbeit.                                                    |
|---------------------------------------------------------------------------------------|
| Anschluss:            | Die DCF-Antenne wird am Port D, Pin 2 (INTO) angeschlossen.   |
|                       | Zustzlich wird auch der Interrupt 0 bentigt.                |
|                       | Von der Antenne ist der >NICHT-invertierender AUSGANG<        |
|                       | zu benutzen!!                                                 |
|-----------------------|---------------------------------------------------------------|
| Peripherie:           |                                                               |
| Timer 0               | Der Timer muss mit einer Taktung von 4MHz laufen. Auch der    |
|                       | "overflow- Interrupt" wird bentigt.                          |
|,,,,,,,,,,,,,,,,,,,,,,,|,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,|
| Interrupt 0           | Die DCF-Antenne wird direkt an dem Interrupt angeschlossen.   |
|                       | Dabei ist darauf zu achten das der Interrupt auf steigende    |
|                       | Flanke reagiert.                                              |
|-----------------------|---------------------------------------------------------------|
****************************************************************************************/


#include <dcf.h>
#include <mega16.h>


unsigned char neusekunde; // |-- Diese Variablen werden bentigt, damit die Uhr auch wirlich
unsigned char altsekunde; // |   nur bei jeder neuen Sekunde nur ein mal auf dem Display
                          //     geschrieben wird.


#asm                      // LDC-Display am PORT C
   .equ __lcd_port=0x15   
#endasm
#include <lcd.h>


/*********************************************************************/
/* Timer_0 Interrupt-Service Routine                                 */
/*                                                                   */
/* Der Timer_0 luft mit einem Takt von 4MHz. Der Timer_0 hat 8 bit, */
/* also 256 Schritte. Somit dauert einen Takt des Timers 64s.       */
/* [ (4MHz/256) = 15625 -> *-1 = 64s]                               */

interrupt [TIM0_OVF] void timer0_ovf_isr(void)
{
 unsigned char *antenne;	// diese Variable bentigt man um den Wert am
			        // PORT D,Pin 2 an eine Funktion
			        // in dcf.h zu bergeben
			
 antenne = PIND.2;              // bekommt den Wert der am Port D Pin 2
 				// anliegt. (DCF_Antenne)

//  |-- diese Variable bekommt den >momentanen< Wert von
//  |   "uhrwerk_sekunde" aus dcf.h! Die Variable wird
//  |   im Hauptprogramm verwendet.
//  |
//  |                            |-- bergabe des Wertes von Port D, Pin 2
//  |                            | 
neusekunde = dcf_uhr_einlesen(antenne);	// Aufruf der Funktion in dcf.h
}
/*********************************************************************/




/*********************************************************************/
/* Interrupt-Service Routine                                         */
/*                                                                   */
/* Dieser Interrupt wird durch die DCF-Antenne ausgelst             */


interrupt [EXT_INT0] void ext_int0_isr(void)
{
 dcf_interrupt_aufruf();	// Aufruf der Funktion in dcf.h
}
/*********************************************************************/




void main(void)
{

// Port D Initialisierung
PORTD=0xff;
DDRD=0x00;

// Timer_0 Initialisierung
TCCR0=0x01;     // 4MHz
TCNT0=0x00;
OCR0=0x00;

// Externer Interrupt INT0
GICR|=0x40;
MCUCR=0x03;	// Steigende Flanke
MCUCSR=0x00;
GIFR=0x40;

// Timer & Interrupt Initialisierung
TIMSK=0x05;

// LCD Module Initialisieren
lcd_init(20);      // LCD mit 20 Zeilen            


/* WICHTIG FR DIE SYMBOLE AUF DEM DISPLAY */
// Laden der Sonderzeichen in das LCD
sonderzeichen_laden();	

// Global enable interrupts
#asm("sei")


while (1)
 {
   if(altsekunde!=neusekunde)     // ist der Wert ungleich, dann war
    { 				  // ein Sekundenwechsel.
     dcf_uhrzeit_anzeigen();	  // Aufruf der Funktion in dcf.h.
     altsekunde = neusekunde;     // die altsekunde neu berschreiben.
    }                              

 
 }; // Ende while
}   // Ende main